@@ -7,19 +7,19 @@ Do not add this file to the repository.
 
 import datetime
 
-version = '1.7'
-date = 'Wed Jul  4 17:53:34 2012'
+version = '1.9'
+date = 'Wed Jun 25 12:02:42 2014'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
 dev = False
 
 # Format: (name, major, min, revision)
-version_info = ('networkx', 1, 7, '70eea5d9e665')
+version_info = ('networkx', '1', '9', None)
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2012, 7, 4, 17, 53, 34, 228318)
+date_info = datetime.datetime(2014, 6, 25, 12, 2, 42, 340629)
 
 # Format: (vcs, vcs_tuple)
-vcs_info = ('mercurial', ('70eea5d9e665', 'tip'))
+vcs_info = (None, (None, None))
 
